package de.ozgcloud.eingang.semantik.enginebased.dfoerdermittel;

import java.io.IOException;
import java.io.InputStream;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Predicate;
import java.util.function.UnaryOperator;
import java.util.stream.Stream;

import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.dataformat.xml.XmlMapper;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormHeader;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.ServiceKonto;
import de.ozgcloud.eingang.common.formdata.ZustaendigeStelle;
import de.ozgcloud.eingang.semantik.common.ServiceKontoFactory;
import de.ozgcloud.eingang.semantik.enginebased.EngineBasedSemantikAdapter;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;

@Log4j2
@Component
public class DFoerdermittelEngineBasedSemantikAdapter implements EngineBasedSemantikAdapter {

	private static final String FACHNACHRICHT_SUFFIX = "Fachnachricht.xml";
	private static final Predicate<IncomingFile> IS_FACHNACHRICHT = inFile -> StringUtils.endsWith(inFile.getName(), FACHNACHRICHT_SUFFIX);

	private static final String KEY_FACHNACHRICHT = "Fachnachricht";
	private static final String KEY_POSTFACH_ID = "InboxReference";
	private static final String KEY_ORGANISATIONS_EINHEIT_ID = "MetaText1";
	private static final String KEY_FORM_NAME = "Title";
	private static final String KEY_FORM_ID = "ProjectTitle";

	static final String DEFAULT_FORM_NAME = "dFördermittelantrag";
	static final String DEFAULT_FORM_ID = "dFördermittelantrag";

	@Autowired
	private ServiceKontoFactory serviceKontoFactory;

	@Override
	public FormData parseFormData(FormData formData) {
		return formData.getRepresentations().stream().filter(IS_FACHNACHRICHT).findAny()
				.map(inFile -> parseFachnachricht(formData, inFile))
				.map(this::processFachnachricht)
				.orElse(formData);
	}

	FormData processFachnachricht(FormData formData) {
		return pipe(
				Stream.of(
						this::addFormName,
						this::addFormEngineName,
						this::addServiceKonto,
						this::addOrganisationsEinheitId,
						this::addFormId
				)
		).apply(formData);
	}

	private UnaryOperator<FormData> pipe(Stream<UnaryOperator<FormData>> functions) {
		return (formData) -> functions
				.reduce(formData, (acc, f) -> f.apply(acc), (a, b) -> a);
	}

	FormData addFormId(FormData formData) {
		return mapWithModifiedHeader(formData, headerBuilder -> headerBuilder.formId(
				getNonEmptyFachnachrichtValueByKey(formData, KEY_FORM_ID)
						.orElse(DEFAULT_FORM_ID)
		));
	}

	private Optional<String> getNonEmptyFachnachrichtValueByKey(FormData formData, String key) {
		return Optional.ofNullable((String) getFachnachricht(formData).get(key))
				.filter(StringUtils::isNotBlank);
	}

	private FormData mapWithModifiedHeader(FormData formData, UnaryOperator<FormHeader.FormHeaderBuilder> headerBuilderOperator) {
		return formData.toBuilder()
				.header(headerBuilderOperator.apply(formData.getHeader().toBuilder()).build())
				.build();
	}

	FormData addServiceKonto(FormData formData) {
		return Optional.ofNullable((String) getFachnachricht(formData).get(KEY_POSTFACH_ID))
				.map(this::extractPrefix)
				.map(this::createServiceKonto)
				.map(serviceKonto -> mapWithModifiedHeader(formData, headerBuilder -> headerBuilder.serviceKonto(serviceKonto)))
				.orElse(formData);
	}

	FormData addOrganisationsEinheitId(FormData formData) {
		return Optional.ofNullable((String) getFachnachricht(formData).get(KEY_ORGANISATIONS_EINHEIT_ID))
				.map(orgaId -> addOrganisationsEinheitId(orgaId, formData.getZustaendigeStelle()))
				.map(zustStelle -> formData.toBuilder().zustaendigeStelle(zustStelle).build())
				.orElse(formData);
	}

	private ZustaendigeStelle addOrganisationsEinheitId(String orgaId, ZustaendigeStelle zustaendigeStelle) {
		ZustaendigeStelle.ZustaendigeStelleBuilder zustaendigeStelleBuilder;
		if (Objects.isNull(zustaendigeStelle)) {
			zustaendigeStelleBuilder = ZustaendigeStelle.builder();
		} else {
			zustaendigeStelleBuilder = zustaendigeStelle.toBuilder();
		}

		return zustaendigeStelleBuilder.organisationseinheitenId(orgaId).build();
	}

	FormData addFormName(FormData formData) {
		return mapWithModifiedHeader(formData, headerBuilder -> headerBuilder.formName(
				getNonEmptyFachnachrichtValueByKey(formData, KEY_FORM_NAME)
						.orElse(DEFAULT_FORM_NAME)
		));
	}

	@SuppressWarnings("unchecked")
	private Map<String, Object> getFachnachricht(FormData formData) {
		return (Map<String, Object>) MapUtils.getMap(formData.getFormData(), KEY_FACHNACHRICHT, Collections.<String, Object>emptyMap());
	}

	FormData addFormEngineName(FormData formData) {
		return mapWithModifiedHeader(formData, headerBuilder -> headerBuilder.formEngineName("dFördermittelantrag"));
	}

	String extractPrefix(@NonNull String postfachId) {
		return postfachId.substring(postfachId.lastIndexOf("/") + 1);
	}

	private ServiceKonto createServiceKonto(String postfachId) {
		return serviceKontoFactory.buildOsiServiceKonto(postfachId);
	}

	FormData parseFachnachricht(FormData formData, IncomingFile fachnachrichtFile) {
		var fachnachrichtData = extractFormDataFormXML(fachnachrichtFile.getContentStream());

		if (MapUtils.isNotEmpty(fachnachrichtData)) {
			var editable = new HashMap<>(formData.getFormData());
			editable.put(KEY_FACHNACHRICHT, fachnachrichtData);
			return formData.toBuilder().formData(Collections.unmodifiableMap(editable)).build();
		}

		return formData;
	}

	@SuppressWarnings("unchecked")
	Map<String, Object> extractFormDataFormXML(InputStream xmlFileStream) {

		XmlMapper xmlMapper = new XmlMapper();
		try {
			return xmlMapper.readValue(xmlFileStream, Map.class);
		} catch (IOException e) {
			LOG.error("Error reading xml fachnachricht.", e);
		}
		return Collections.emptyMap();
	}

	@Override
	public boolean isResponsible(FormData formData) {
		return formData.getRepresentations().stream().anyMatch(IS_FACHNACHRICHT);
	}

}
