/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch das
 * Ministerium für Energiewende, Klimaschutz, Umwelt und Natur
 * Zentrales IT-Management
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.eingang.xta;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.UUID;
import java.util.stream.Stream;

import de.ozgcloud.eingang.common.formdata.IncomingFileGroup;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.IncomingFileTestFactory;
import de.ozgcloud.eingang.xta.zip.ZipFileExtractor;

class XtaIncomingFilesMapperTest {
	@Spy
	@InjectMocks
	private XtaIncomingFilesMapper mapper;

	@Mock
	private ZipFileExtractor extractor;

	private static final String ZIP_CONTENT_TYPE = "application/zip";

	@Nested
	class TestToIncomingFiles {

		@Test
		void shouldMapToIncomingFiles() {
			var xtaFile = XtaFileTestFactory.create();
			var incomingFile = IncomingFileTestFactory.create();
			when(mapper.toIncomingFile(xtaFile)).thenReturn(incomingFile);
			when(mapper.extractZip(incomingFile)).thenAnswer(x -> Stream.of(incomingFile));

			mapper.toIncomingFiles(List.of(xtaFile, xtaFile));

			inOrder(mapper).verify(mapper, calls(2)).toIncomingFile(xtaFile);
			inOrder(mapper).verify(mapper, calls(2)).extractZip(incomingFile);
		}

		@Test
		void shouldHandleMissingMessageFile() {
			var fileGroup = mapper.toIncomingFiles(null);

			assertThat(fileGroup).isEmpty();
		}
	}

	@Nested
	class ToIncomingFile {
		@Test
		void shouldHaveMessageFile() {
			var inFile = mapper.toIncomingFile(XtaFileTestFactory.create());

			assertThat(inFile.getContentStream()).isNotNull();
		}

		@Test
		void shouldHaveFileName() {
			var inFile = mapper.toIncomingFile(XtaFileTestFactory.create());

			assertThat(inFile.getName()).isEqualTo(XtaFileTestFactory.NAME);
		}

		@Test
		void shouldHaveZipContentType() {
			var inFile = mapper.toIncomingFile(XtaFileTestFactory.create());

			assertThat(inFile.getContentType()).isEqualTo(XtaFileTestFactory.ZIP_CONTENT_TYPE);
		}

		@Test
		void shouldHaveSize() {
			var inFile = mapper.toIncomingFile(XtaFileTestFactory.create());

			assertThat(inFile.getSize()).isEqualTo(XtaFileTestFactory.CONTENT.length());
		}
	}

	@Nested
	class TestExtractZip {

		@Mock
		IncomingFile outFile1;

		@Mock
		IncomingFile outFile2;


		@Test
		void shouldExtractZipFiles() {
			var expectedExtractedFiles = List.of(outFile1, outFile2);
			var zipFile = createTestIncomingFile();
			when(extractor.extractIncomingFilesSafely(zipFile)).thenReturn(expectedExtractedFiles);

			var extractedFiles = mapper.extractZip(zipFile).toList();

			assertThat(extractedFiles).isEqualTo(expectedExtractedFiles);
		}

		IncomingFile createTestIncomingFile() {
			return IncomingFileTestFactory.createBuilder()
					.name("attachments.zip")
					.contentType(ZIP_CONTENT_TYPE)
					.build();
		}

		@Test
		void shouldIgnoreNonZipFiles() {
			var incomingFile = IncomingFileTestFactory.create();

			var extractedFiles = mapper.extractZip(incomingFile).toList();

			assertThat(extractedFiles).containsExactly(incomingFile);
		}
	}

	@Test
	void testIsZipFilePredicate() {
		assertThat(XtaIncomingFilesMapper.IS_ZIP_FILE.test(IncomingFileTestFactory.create())).isFalse();
		assertThat(XtaIncomingFilesMapper.IS_ZIP_FILE.test(IncomingFileTestFactory.createBuilder().contentType(ZIP_CONTENT_TYPE).build()))
				.isTrue();
	}

}
